import numpy as np

def calcArrayManifoldVector(iElNum, pfElPos, eBfType, pfFreqVec, pfThetaRad, fSoundspeed, eTxOrRxBf):
    # Calculates and returns array manifold vector and the beampattern based on the given coefficients
    # and microphone positions.

    # Create empty element coefficients
    pfElCoe = np.zeros((iElNum,pfFreqVec.shape[0]))

    # Fill element coefficients dependent on bf type
    if eBfType == 'delsum':
        pfElCoe = 1 / iElNum * np.ones((iElNum,pfFreqVec.shape[0]))
    elif eBfType == 'blocking':
        if pfElCoe.shape[0] > 1:
            pfElCoe[0] = 1
            pfElCoe[1] = -1
    else:
        raise ValueError('Beamformer type not supported')

    # Fill element coefficients dependent on bf type
    if eTxOrRxBf == 'tx':
        fSign = 1
        pfElCoe = pfElCoe * iElNum
    elif eTxOrRxBf == 'rx':
        fSign = -1
    else:
        raise ValueError('Beamformer type not supported')

    # Calculate tau for each microphone and each possible frequency (Slide 10.)
    pppfTau = np.zeros((pfThetaRad.shape[0], pfElPos.shape[0], pfFreqVec.shape[0]))
    for iTheta in np.arange(pfThetaRad.shape[0]):
        for iElPos in np.arange(pfElPos.shape[0]):
            for iFreq in np.arange(pfFreqVec.shape[0]):
                pppfTau[iTheta,iElPos,iFreq] = ### ADD SOME CODE HERE ###

    # Calculate complex delays
    pppcArrayManifoldVector = np.zeros((pfThetaRad.shape[0], pfElPos.shape[0], pfFreqVec.shape[0]),dtype=complex)
    for iTheta in np.arange(pfThetaRad.shape[0]):
        for iElPos in np.arange(pfElPos.shape[0]):
            for iFreq in np.arange(pfFreqVec.shape[0]):
                pppcArrayManifoldVector[iTheta,iElPos,iFreq] = ### ADD SOME CODE HERE ###

    # Calculate beampattern for plotting (summation over all microphones)
    pppcBeampattern = np.sum(pppcArrayManifoldVector, axis=1)

    return pppcBeampattern, pppcArrayManifoldVector