import numpy as np

def calcArrayManifoldVectorSln(iElNum, pfElPos, eBfType, pfFreqVec, pfThetaRad, fSoundspeed, eTxOrRxBf):
    # Calculates and returns array manifold vector and the beampattern based on the given coefficients
    # and microphone positions.

    # Create empty element coefficients
    pfElCoe = np.zeros((iElNum,pfFreqVec.shape[0]))

    # Fill element coefficients dependent on bf type
    if eBfType == 'delsum':
        pfElCoe = 1 / iElNum * np.ones((iElNum,pfFreqVec.shape[0]))
    elif eBfType == 'blocking':
        if pfElCoe.shape[0] > 1:
            pfElCoe[0] = 1
            pfElCoe[1] = -1
    else:
        raise ValueError('Beamformer type not supported')

    # Fill element coefficients dependent on bf type
    if eTxOrRxBf == 'tx':
        fSign = 1
        pfElCoe = pfElCoe * iElNum
    elif eTxOrRxBf == 'rx':
        fSign = -1
    else:
        raise ValueError('Beamformer type not supported')

    # Calculate tau for each microphone and each possible frequency (Slide 10.)
    pppfTau = np.zeros((pfThetaRad.shape[0], pfElPos.shape[0], pfFreqVec.shape[0]))
    for iTheta in np.arange(pfThetaRad.shape[0]):
        for iElPos in np.arange(pfElPos.shape[0]):
            for iFreq in np.arange(pfFreqVec.shape[0]):
                pppfTau[iTheta,iElPos,iFreq] = np.sin(pfThetaRad[iTheta]) * pfElPos[iElPos] * pfFreqVec[iFreq] / fSoundspeed
    
    # A faster way in one single line (feel free to look into the used code optimization with np.outer() and the [:,:,None] statement)
    pppfTauFast = np.outer(np.sin(pfThetaRad), pfElPos)[:,:,None] * pfFreqVec / fSoundspeed

    # Calculate complex delays
    pppcArrayManifoldVector = np.zeros((pfThetaRad.shape[0], pfElPos.shape[0], pfFreqVec.shape[0]),dtype=complex)
    for iTheta in np.arange(pfThetaRad.shape[0]):
        for iElPos in np.arange(pfElPos.shape[0]):
            for iFreq in np.arange(pfFreqVec.shape[0]):
                pppcArrayManifoldVector[iTheta,iElPos,iFreq] = pfElCoe[iElPos,iFreq] * np.exp(fSign * 1j * 2 * np.pi * pppfTau[iTheta,iElPos,iFreq])

    # A faster way in one single line (feel free to look into the used code optimization with np.exp() of matrices and the [None,:,:] statement)
    pppcArrayManifoldVectorFast = pfElCoe[None,:,:] * np.exp(fSign * 1j * 2 * np.pi * pppfTau) ### ADD SOME CODE HERE ###

    # Calculate beampattern for plotting (summation over all microphones)
    pppcBeampattern = np.sum(pppcArrayManifoldVector, axis=1)

    return pppcBeampattern, pppcArrayManifoldVector