import numpy as np
import soundfile as sf
from scipy.signal import stft
from scipy.signal import istft

from calc_array_manifold_vector_solution import calcArrayManifoldVectorSln

def createIncomingWavFiles(iFrameshift, iNfft, iElNum, iIncomingIndex, iNoiseDirIndex, pfElPos, pfFreqVec, fThetaRad, fSoundspeed):

    # Calculate array manifold vector and beampattern
    pppcBeampatternTx, pppcArrayManifoldVectorTx = calcArrayManifoldVectorSln(iElNum, pfElPos, 'delsum', pfFreqVec, fThetaRad, fSoundspeed, 'tx')

    # STFT of observed signal
    iFileLengthInSamp = len(sf.SoundFile('speech.wav')) 
    pfNoise = 0.1 * np.random.normal(size=iFileLengthInSamp)
    pppfSignalDes = np.zeros((iElNum, iFileLengthInSamp))
    pppfSignalNoise = np.zeros((iElNum, iFileLengthInSamp))
    ppcSignalDes = np.zeros((iElNum, int(iNfft/2+1), int(np.ceil(iFileLengthInSamp / iFrameshift ) + 1)), dtype=np.complex_)
    ppcSignalNoise = np.zeros((iElNum, int(iNfft/2+1), int(np.ceil(iFileLengthInSamp / iFrameshift ) + 1)), dtype=np.complex_)
    for iEl in np.arange(iElNum):
        pppfSignalDes[iEl,:], fSamplerate = sf.read('speech.wav')
        pppfSignalNoise[iEl,:] = pfNoise
        f_y, t_y, ppcSignalDes[iEl,:,:] = stft(pppfSignalDes[iEl,:], fs=fSamplerate, nperseg=iNfft, noverlap=iNfft-iFrameshift)
        f_y, t_y, ppcSignalNoise[iEl,:,:] = stft(pppfSignalNoise[iEl,:], fs=fSamplerate, nperseg=iNfft, noverlap=iNfft-iFrameshift)

    # Compute output spectrum
    ppcSignalBfDes = np.zeros(np.shape(ppcSignalDes), dtype=np.complex_)
    ppcSignalBfNoise = np.zeros(np.shape(ppcSignalDes), dtype=np.complex_)
    for iEl in np.arange(iElNum):
        for iFrame in np.arange(t_y.shape[0]):
            ppcSignalBfDes[iEl,:,iFrame] = np.multiply(pppcArrayManifoldVectorTx[iIncomingIndex,iEl,:],ppcSignalDes[iEl,:,iFrame])
            ppcSignalBfNoise[iEl,:,iFrame] = np.multiply(pppcArrayManifoldVectorTx[iNoiseDirIndex,iEl,:],ppcSignalNoise[iEl,:,iFrame])

    # Synthesis
    ppfSignalBfDes = np.zeros((iElNum, int(np.ceil(iFileLengthInSamp / iFrameshift )) * iFrameshift))
    ppfSignalBfNoise = np.zeros((iElNum, int(np.ceil(iFileLengthInSamp / iFrameshift )) * iFrameshift))
    for iEl in np.arange(iElNum):
        t_y, ppfSignalBfDes[iEl,:] = istft(ppcSignalBfDes[iEl,:,:], fs=fSamplerate, nperseg=iNfft, noverlap=iNfft-iFrameshift)
        t_y, ppfSignalBfNoise[iEl,:] = istft(ppcSignalBfNoise[iEl,:,:], fs=fSamplerate, nperseg=iNfft, noverlap=iNfft-iFrameshift)

    # Save .wav files
    ppfSignalBf = np.zeros(ppfSignalBfDes.shape)
    for iEl in np.arange(iElNum):
        ppfSignalBf[iEl,:] = ppfSignalBfDes[iEl,:] + ppfSignalBfNoise[iEl,:]
        sf.write('bf_' + str(iEl) + '.wav', ppfSignalBf[iEl,:], fSamplerate)